#ifndef __DBUS_ADAPTER_CONEXUS_DBUS_FILE_H
#define __DBUS_ADAPTER_CONEXUS_DBUS_FILE_H

#include <dbus-cxx.h>

#include <conexus/file.h>

#include <conexus-dbus/filedescriptor_adapter.h>


namespace Conexus {
  namespace DBus {

    class FileAdapter : public FileDescriptorAdapter
    {
      protected:
        FileAdapter( Conexus::File* adaptee=NULL, const std::string& path="/Conexus/File"):
          FileDescriptorAdapter(adaptee, path),
          m_adaptee(adaptee)
        {

          ::DBus::MethodBase::pointer temp_method;

          temp_method = this->create_method<void,std::string,uint64_t>( "Conexus.File", "open", sigc::mem_fun(*this, &FileAdapter::open_adapter_stub_vst) );
          temp_method->set_arg_name( 1, "filename" );
          temp_method->set_arg_name( 2, "mode" );
          temp_method = this->create_method<std::string>( "Conexus.File", "filename", sigc::mem_fun(*this, &FileAdapter::filename_adapter_stub_s) );
          temp_method->set_arg_name( 0, "f" );
          temp_method = this->create_method<uint64_t>( "Conexus.File", "mode", sigc::mem_fun(*this, &FileAdapter::mode_adapter_stub_t) );
          temp_method->set_arg_name( 0, "m" );
          temp_method = this->create_method<bool>( "Conexus.File", "eof", sigc::mem_fun(*this, &FileAdapter::eof_adapter_stub_b) );
          temp_method->set_arg_name( 0, "state" );
          temp_method = this->create_method<void,bool>( "Conexus.File", "set_eof", sigc::mem_fun(*this, &FileAdapter::set_eof_adapter_stub_vb) );
          temp_method->set_arg_name( 1, "state" );
          temp_method = this->create_method<void,std::string>( "Conexus.File", "set_filename", sigc::mem_fun(*this, &FileAdapter::set_filename_adapter_stub_vs) );
          temp_method->set_arg_name( 1, "new_filename" );
          temp_method = this->create_method<void,std::string,uint64_t>( "Conexus.File", "set_filename", sigc::mem_fun(*this, &FileAdapter::set_filename_adapter_stub_vst) );
          temp_method->set_arg_name( 1, "new_filename" );
          temp_method->set_arg_name( 2, "mode" );
          m_signal_adapter_filename_changed = this->create_signal<void>("Conexus.File","filename_changed");
          if ( m_adaptee ) m_signal_adapter_connection_filename_changed = m_adaptee->signal_filename_changed().connect( *m_signal_adapter_filename_changed );
        }

      public:

        typedef DBusCxxPointer<FileAdapter> pointer;

        static pointer create( Conexus::File* adaptee=NULL, const std::string& path="/Conexus/File")
          { return pointer( new FileAdapter(adaptee, path)); }

        Conexus::File* adaptee() { return m_adaptee; }

        void set_adaptee( Conexus::File* adaptee ) {
          m_adaptee = adaptee;
          m_signal_adapter_connection_filename_changed.disconnect();
          if ( m_adaptee ) m_signal_adapter_connection_filename_changed = m_adaptee->signal_filename_changed().connect( *m_signal_adapter_filename_changed );
        }

        void set_adaptee( Conexus::File& adaptee ) {
          this->set_adaptee(&adaptee);
        }


      private:

        Conexus::File* m_adaptee;


        void check_adaptee() { if ( not m_adaptee) throw ::DBus::ErrorInvalidAdaptee::create(); }

        void open_adapter_stub_vst( std::string filename, uint64_t mode ) { this->check_adaptee(); m_adaptee->open( filename, mode); }
        std::string filename_adapter_stub_s( ) { this->check_adaptee(); return m_adaptee->filename(); }
        uint64_t mode_adapter_stub_t( ) { this->check_adaptee(); return m_adaptee->mode(); }
        bool eof_adapter_stub_b( ) { this->check_adaptee(); return m_adaptee->eof(); }
        void set_eof_adapter_stub_vb( bool state ) { this->check_adaptee(); m_adaptee->set_eof( state); }
        void set_filename_adapter_stub_vs( std::string new_filename ) { this->check_adaptee(); m_adaptee->set_filename( new_filename); }
        void set_filename_adapter_stub_vst( std::string new_filename, uint64_t mode ) { this->check_adaptee(); m_adaptee->set_filename( new_filename, mode); }
        ::DBus::signal<void>::pointer m_signal_adapter_filename_changed;
        sigc::connection m_signal_adapter_connection_filename_changed;


    };

  }
}

#endif
