#ifndef __DBUS_PROXY_CONEXUS_DBUS_FILE_H
#define __DBUS_PROXY_CONEXUS_DBUS_FILE_H

#include <dbus-cxx.h>

#include <conexus/file.h>

#include <conexus-dbus/filedescriptor_proxy.h>


namespace Conexus {
  namespace DBus {

    class FileProxy : public FileDescriptorProxy
    {
      protected:
        FileProxy( ::DBus::Connection::pointer conn, const std::string& dest="Conexus.File", const std::string& path="/Conexus/File"):
          FileDescriptorProxy(conn, dest, path)
        {
          ::DBus::Path child_path;

          m_method_open_vst = this->create_method< void,std::string,uint64_t >( "Conexus.File", "open" );
          m_method_filename_s = this->create_method< std::string >( "Conexus.File", "filename" );
          m_method_mode_t = this->create_method< uint64_t >( "Conexus.File", "mode" );
          m_method_eof_b = this->create_method< bool >( "Conexus.File", "eof" );
          m_method_set_eof_vb = this->create_method< void,bool >( "Conexus.File", "set_eof" );
          m_method_set_filename_vs = this->create_method< void,std::string >( "Conexus.File", "set_filename" );
          m_method_set_filename_vst = this->create_method< void,std::string,uint64_t >( "Conexus.File", "set_filename" );
          m_signal_filename_changed = this->create_signal<void>( "Conexus.File", "filename_changed" );
        }

      public:

        typedef DBusCxxPointer<FileProxy> pointer;

        static pointer create( ::DBus::Connection::pointer conn,
                               const std::string& dest="Conexus.File",
                               const std::string& path="/Conexus/File"
                             )
        { return pointer( new FileProxy(conn, dest, path)); }


        void open( std::string filename, uint64_t mode ) { return (*m_method_open_vst)( filename, mode); }

        std::string filename( ) { return (*m_method_filename_s)(); }

        uint64_t mode( ) { return (*m_method_mode_t)(); }

        bool eof( ) { return (*m_method_eof_b)(); }

        void set_eof( bool state ) { return (*m_method_set_eof_vb)( state); }

        void set_filename( std::string new_filename ) { return (*m_method_set_filename_vs)( new_filename); }

        void set_filename( std::string new_filename, uint64_t mode ) { return (*m_method_set_filename_vst)( new_filename, mode); }

        ::DBus::signal_proxy<void >& signal_filename_changed() { return *m_signal_filename_changed; }


      protected:


        ::DBus::MethodProxy<void,std::string,uint64_t>::pointer m_method_open_vst;
        ::DBus::MethodProxy<std::string>::pointer m_method_filename_s;
        ::DBus::MethodProxy<uint64_t>::pointer m_method_mode_t;
        ::DBus::MethodProxy<bool>::pointer m_method_eof_b;
        ::DBus::MethodProxy<void,bool>::pointer m_method_set_eof_vb;
        ::DBus::MethodProxy<void,std::string>::pointer m_method_set_filename_vs;
        ::DBus::MethodProxy<void,std::string,uint64_t>::pointer m_method_set_filename_vst;
        ::DBus::signal_proxy<void>::pointer m_signal_filename_changed;


    };

  }
}

#endif
