#ifndef __DBUS_PROXY_CONEXUS_DBUS_IPV4_ADDRESS_H
#define __DBUS_PROXY_CONEXUS_DBUS_IPV4_ADDRESS_H

#include <dbus-cxx.h>

#include <conexus/ipv4_address.h>

#include <conexus/ipaddress_interface.h>

namespace Conexus {
  namespace DBus {
    namespace IPv4 {

      class AddressProxy : public ::DBus::ObjectProxy, public virtual IPAddressInterface
      {
        protected:
          AddressProxy( ::DBus::Connection::pointer conn, const std::string& dest="Conexus.IPv4.Address", const std::string& path="/Conexus/IPv4/Address"):
            ::DBus::ObjectProxy(conn, dest, path)
          {
            ::DBus::Path child_path;

            m_method_host_u = this->create_method< uint32_t >( "Conexus.IPv4.Address", "host" );
            m_method_host_string_s = this->create_method< std::string >( "Conexus.IPv4.Address", "host_string" );
            m_method_subnet_mask_u = this->create_method< uint32_t >( "Conexus.IPv4.Address", "subnet_mask" );
            m_method_subnet_mask_string_s = this->create_method< std::string >( "Conexus.IPv4.Address", "subnet_mask_string" );
            m_method_prefix_length_u = this->create_method< uint32_t >( "Conexus.IPv4.Address", "prefix_length" );
            m_method_prefix_length_string_s = this->create_method< std::string >( "Conexus.IPv4.Address", "prefix_length_string" );
            m_method_host_subnet_mask_string_s = this->create_method< std::string >( "Conexus.IPv4.Address", "host_subnet_mask_string" );
            m_method_cidr_host_string_s = this->create_method< std::string >( "Conexus.IPv4.Address", "cidr_host_string" );
            m_method_prefix_u = this->create_method< uint32_t >( "Conexus.IPv4.Address", "prefix" );
            m_method_prefix_string_s = this->create_method< std::string >( "Conexus.IPv4.Address", "prefix_string" );
            m_method_broadcast_address_u = this->create_method< uint32_t >( "Conexus.IPv4.Address", "broadcast_address" );
            m_method_broadcast_address_string_s = this->create_method< std::string >( "Conexus.IPv4.Address", "broadcast_address_string" );
            m_method_local_address_u = this->create_method< uint32_t >( "Conexus.IPv4.Address", "local_address" );
            m_method_local_address_string_s = this->create_method< std::string >( "Conexus.IPv4.Address", "local_address_string" );
            m_method_port_q = this->create_method< uint16_t >( "Conexus.IPv4.Address", "port" );
            m_method_port_string_s = this->create_method< std::string >( "Conexus.IPv4.Address", "port_string" );
            m_method_set_host_vu = this->create_method< void,uint32_t >( "Conexus.IPv4.Address", "set_host" );
            m_method_set_host_vs = this->create_method< void,const std::string& >( "Conexus.IPv4.Address", "set_host" );
            m_method_set_host_vsq = this->create_method< void,const std::string&,uint16_t >( "Conexus.IPv4.Address", "set_host" );
            m_method_set_host_subnet_mask_vuu = this->create_method< void,uint32_t,uint32_t >( "Conexus.IPv4.Address", "set_host_subnet_mask" );
            m_method_set_host_prefix_length_vuu = this->create_method< void,uint32_t,uint32_t >( "Conexus.IPv4.Address", "set_host_prefix_length" );
            m_method_set_subnet_mask_vu = this->create_method< void,uint32_t >( "Conexus.IPv4.Address", "set_subnet_mask" );
            m_method_set_subnet_mask_vs = this->create_method< void,std::string >( "Conexus.IPv4.Address", "set_subnet_mask" );
            m_method_set_prefix_length_vu = this->create_method< void,uint32_t >( "Conexus.IPv4.Address", "set_prefix_length" );
            m_method_set_prefix_length_vs = this->create_method< void,const std::string& >( "Conexus.IPv4.Address", "set_prefix_length" );
            m_method_set_port_vq = this->create_method< void,uint16_t >( "Conexus.IPv4.Address", "set_port" );
            m_method_set_port_vs = this->create_method< void,const std::string& >( "Conexus.IPv4.Address", "set_port" );
            m_method_is_valid_hostname_bs = this->create_method< bool,std::string >( "Conexus.IPv4.Address", "is_valid_hostname" );
            m_method_hostname_s = this->create_method< std::string >( "Conexus.IPv4.Address", "hostname" );
            m_method_servicename_s = this->create_method< std::string >( "Conexus.IPv4.Address", "servicename" );
            m_method_is_private_b = this->create_method< bool >( "Conexus.IPv4.Address", "is_private" );
            m_method_is_reserved_b = this->create_method< bool >( "Conexus.IPv4.Address", "is_reserved" );
            m_method_is_loopback_b = this->create_method< bool >( "Conexus.IPv4.Address", "is_loopback" );
            m_method_is_broadcast_b = this->create_method< bool >( "Conexus.IPv4.Address", "is_broadcast" );
            m_method_is_multicast_b = this->create_method< bool >( "Conexus.IPv4.Address", "is_multicast" );
            m_method_is_any_b = this->create_method< bool >( "Conexus.IPv4.Address", "is_any" );
            m_method_is_ipv4_b = this->create_method< bool >( "Conexus.IPv4.Address", "is_ipv4" );
            m_method_is_ipv6_b = this->create_method< bool >( "Conexus.IPv4.Address", "is_ipv6" );
            m_signal_changed = this->create_signal<void>( "Conexus.IPv4.Address", "changed" );
            m_signal_host_changed = this->create_signal<void>( "Conexus.IPv4.Address", "host_changed" );
            m_signal_port_changed = this->create_signal<void>( "Conexus.IPv4.Address", "port_changed" );
            m_signal_prefix_changed = this->create_signal<void>( "Conexus.IPv4.Address", "prefix_changed" );
          }

        public:

          typedef DBusCxxPointer<AddressProxy> pointer;

          static pointer create( ::DBus::Connection::pointer conn,
                                 const std::string& dest="Conexus.IPv4.Address",
                                 const std::string& path="/Conexus/IPv4/Address"
                               )
          { return pointer( new AddressProxy(conn, dest, path)); }


          uint32_t host( ) { return (*m_method_host_u)(); }

          virtual std::string host_string( ) const { return (*m_method_host_string_s)(); }

          uint32_t subnet_mask( ) { return (*m_method_subnet_mask_u)(); }

          virtual std::string subnet_mask_string( ) const { return (*m_method_subnet_mask_string_s)(); }

          virtual uint32_t prefix_length( ) const { return (*m_method_prefix_length_u)(); }

          virtual std::string prefix_length_string( ) const { return (*m_method_prefix_length_string_s)(); }

          virtual std::string host_subnet_mask_string( ) const { return (*m_method_host_subnet_mask_string_s)(); }

          virtual std::string cidr_host_string( ) const { return (*m_method_cidr_host_string_s)(); }

          uint32_t prefix( ) { return (*m_method_prefix_u)(); }

          virtual std::string prefix_string( ) const { return (*m_method_prefix_string_s)(); }

          uint32_t broadcast_address( ) { return (*m_method_broadcast_address_u)(); }

          std::string broadcast_address_string( ) { return (*m_method_broadcast_address_string_s)(); }

          uint32_t local_address( ) { return (*m_method_local_address_u)(); }

          virtual std::string local_address_string( ) const { return (*m_method_local_address_string_s)(); }

          virtual uint16_t port( ) const { return (*m_method_port_q)(); }

          virtual std::string port_string( ) const { return (*m_method_port_string_s)(); }

          void set_host( uint32_t host ) { return (*m_method_set_host_vu)( host); }

          void set_host( const std::string& host ) { return (*m_method_set_host_vs)( host); }

          void set_host( const std::string& host, uint16_t port ) { return (*m_method_set_host_vsq)( host, port); }

          void set_host_subnet_mask( uint32_t host, uint32_t subnet_mask ) { return (*m_method_set_host_subnet_mask_vuu)( host, subnet_mask); }

          void set_host_prefix_length( uint32_t host, uint32_t prefix_length ) { return (*m_method_set_host_prefix_length_vuu)( host, prefix_length); }

          void set_subnet_mask( uint32_t subnet_mask ) { return (*m_method_set_subnet_mask_vu)( subnet_mask); }

          void set_subnet_mask( std::string subnet_mask ) { return (*m_method_set_subnet_mask_vs)( subnet_mask); }

          void set_prefix_length( uint32_t prefix_length ) { return (*m_method_set_prefix_length_vu)( prefix_length); }

          void set_prefix_length( const std::string& prefix_length ) { return (*m_method_set_prefix_length_vs)( prefix_length); }

          void set_port( uint16_t port ) { return (*m_method_set_port_vq)( port); }

          void set_port( const std::string& port ) { return (*m_method_set_port_vs)( port); }

          bool is_valid_hostname( std::string name ) { return (*m_method_is_valid_hostname_bs)( name); }

          virtual std::string hostname( ) const { return (*m_method_hostname_s)(); }

          virtual std::string servicename( ) const { return (*m_method_servicename_s)(); }

          bool is_private( ) { return (*m_method_is_private_b)(); }

          bool is_reserved( ) { return (*m_method_is_reserved_b)(); }

          virtual bool is_loopback( ) const { return (*m_method_is_loopback_b)(); }

          bool is_broadcast( ) { return (*m_method_is_broadcast_b)(); }

          virtual bool is_multicast( ) const { return (*m_method_is_multicast_b)(); }

          virtual bool is_any( ) const { return (*m_method_is_any_b)(); }

          bool is_ipv4( ) { return (*m_method_is_ipv4_b)(); }

          bool is_ipv6( ) { return (*m_method_is_ipv6_b)(); }

          ::DBus::signal_proxy<void >& signal_changed() { return *m_signal_changed; }

          ::DBus::signal_proxy<void >& signal_host_changed() { return *m_signal_host_changed; }

          ::DBus::signal_proxy<void >& signal_port_changed() { return *m_signal_port_changed; }

          ::DBus::signal_proxy<void >& signal_prefix_changed() { return *m_signal_prefix_changed; }


        protected:


          ::DBus::MethodProxy<uint32_t>::pointer m_method_host_u;
          ::DBus::MethodProxy<std::string>::pointer m_method_host_string_s;
          ::DBus::MethodProxy<uint32_t>::pointer m_method_subnet_mask_u;
          ::DBus::MethodProxy<std::string>::pointer m_method_subnet_mask_string_s;
          ::DBus::MethodProxy<uint32_t>::pointer m_method_prefix_length_u;
          ::DBus::MethodProxy<std::string>::pointer m_method_prefix_length_string_s;
          ::DBus::MethodProxy<std::string>::pointer m_method_host_subnet_mask_string_s;
          ::DBus::MethodProxy<std::string>::pointer m_method_cidr_host_string_s;
          ::DBus::MethodProxy<uint32_t>::pointer m_method_prefix_u;
          ::DBus::MethodProxy<std::string>::pointer m_method_prefix_string_s;
          ::DBus::MethodProxy<uint32_t>::pointer m_method_broadcast_address_u;
          ::DBus::MethodProxy<std::string>::pointer m_method_broadcast_address_string_s;
          ::DBus::MethodProxy<uint32_t>::pointer m_method_local_address_u;
          ::DBus::MethodProxy<std::string>::pointer m_method_local_address_string_s;
          ::DBus::MethodProxy<uint16_t>::pointer m_method_port_q;
          ::DBus::MethodProxy<std::string>::pointer m_method_port_string_s;
          ::DBus::MethodProxy<void,uint32_t>::pointer m_method_set_host_vu;
          ::DBus::MethodProxy<void,const std::string&>::pointer m_method_set_host_vs;
          ::DBus::MethodProxy<void,const std::string&,uint16_t>::pointer m_method_set_host_vsq;
          ::DBus::MethodProxy<void,uint32_t,uint32_t>::pointer m_method_set_host_subnet_mask_vuu;
          ::DBus::MethodProxy<void,uint32_t,uint32_t>::pointer m_method_set_host_prefix_length_vuu;
          ::DBus::MethodProxy<void,uint32_t>::pointer m_method_set_subnet_mask_vu;
          ::DBus::MethodProxy<void,std::string>::pointer m_method_set_subnet_mask_vs;
          ::DBus::MethodProxy<void,uint32_t>::pointer m_method_set_prefix_length_vu;
          ::DBus::MethodProxy<void,const std::string&>::pointer m_method_set_prefix_length_vs;
          ::DBus::MethodProxy<void,uint16_t>::pointer m_method_set_port_vq;
          ::DBus::MethodProxy<void,const std::string&>::pointer m_method_set_port_vs;
          ::DBus::MethodProxy<bool,std::string>::pointer m_method_is_valid_hostname_bs;
          ::DBus::MethodProxy<std::string>::pointer m_method_hostname_s;
          ::DBus::MethodProxy<std::string>::pointer m_method_servicename_s;
          ::DBus::MethodProxy<bool>::pointer m_method_is_private_b;
          ::DBus::MethodProxy<bool>::pointer m_method_is_reserved_b;
          ::DBus::MethodProxy<bool>::pointer m_method_is_loopback_b;
          ::DBus::MethodProxy<bool>::pointer m_method_is_broadcast_b;
          ::DBus::MethodProxy<bool>::pointer m_method_is_multicast_b;
          ::DBus::MethodProxy<bool>::pointer m_method_is_any_b;
          ::DBus::MethodProxy<bool>::pointer m_method_is_ipv4_b;
          ::DBus::MethodProxy<bool>::pointer m_method_is_ipv6_b;
          ::DBus::signal_proxy<void>::pointer m_signal_changed;
          ::DBus::signal_proxy<void>::pointer m_signal_host_changed;
          ::DBus::signal_proxy<void>::pointer m_signal_port_changed;
          ::DBus::signal_proxy<void>::pointer m_signal_prefix_changed;


      };

    }
  }
}

#endif
