#ifndef __DBUS_PROXY_CONEXUS_DBUS_IPV4_IP_H
#define __DBUS_PROXY_CONEXUS_DBUS_IPV4_IP_H

#include <dbus-cxx.h>

#include <conexus/ipv4_ip.h>

#include <conexus-dbus/endpoint_proxy.h>

#include <conexus-dbus/ipv4_address_proxy.h>
#include <conexus-dbus/ipv4_address_proxy.h>

namespace Conexus {
  namespace DBus {
    namespace IPv4 {

      class IPProxy : public EndpointProxy
      {
        protected:
          IPProxy( ::DBus::Connection::pointer conn, const std::string& dest="Conexus.IPv4.IP", const std::string& path="/Conexus/IPv4/IP"):
            EndpointProxy(conn, dest, path)
          {
            ::DBus::Path child_path;
            child_path = path; 
            child_path.append_element("LocalAddress");
            m_LocalAddress = AddressProxy::create(conn, dest, child_path);
            child_path = path; 
            child_path.append_element("RemoteAddress");
            m_RemoteAddress = AddressProxy::create(conn, dest, child_path);

            m_method_bind_v = this->create_method< void >( "Conexus.IPv4.IP", "bind" );
            m_method_connect_v = this->create_method< void >( "Conexus.IPv4.IP", "connect" );
            m_method_set_remote_address_vuq = this->create_method< void,uint32_t,uint16_t >( "Conexus.IPv4.IP", "set_remote_address" );
            m_method_set_remote_address_vsq = this->create_method< void,std::string,uint16_t >( "Conexus.IPv4.IP", "set_remote_address" );
            m_method_unset_remote_address_v = this->create_method< void >( "Conexus.IPv4.IP", "unset_remote_address" );
            m_method_set_local_address_vuq = this->create_method< void,uint32_t,uint16_t >( "Conexus.IPv4.IP", "set_local_address" );
            m_method_set_local_address_vsq = this->create_method< void,std::string,uint16_t >( "Conexus.IPv4.IP", "set_local_address" );
          }

        public:

          typedef DBusCxxPointer<IPProxy> pointer;

          static pointer create( ::DBus::Connection::pointer conn,
                                 const std::string& dest="Conexus.IPv4.IP",
                                 const std::string& path="/Conexus/IPv4/IP"
                               )
          { return pointer( new IPProxy(conn, dest, path)); }

          AddressProxy::pointer local_address() { return m_LocalAddress; }
          AddressProxy::pointer remote_address() { return m_RemoteAddress; }

          void bind( ) { return (*m_method_bind_v)(); }

          void connect( ) { return (*m_method_connect_v)(); }

          void set_remote_address( uint32_t addr, uint16_t port ) { return (*m_method_set_remote_address_vuq)( addr, port); }

          void set_remote_address( std::string addr, uint16_t port ) { return (*m_method_set_remote_address_vsq)( addr, port); }

          void unset_remote_address( ) { return (*m_method_unset_remote_address_v)(); }

          void set_local_address( uint32_t addr, uint16_t port ) { return (*m_method_set_local_address_vuq)( addr, port); }

          void set_local_address( std::string addr, uint16_t port ) { return (*m_method_set_local_address_vsq)( addr, port); }


        protected:

          AddressProxy::pointer m_LocalAddress;
          AddressProxy::pointer m_RemoteAddress;

          ::DBus::MethodProxy<void>::pointer m_method_bind_v;
          ::DBus::MethodProxy<void>::pointer m_method_connect_v;
          ::DBus::MethodProxy<void,uint32_t,uint16_t>::pointer m_method_set_remote_address_vuq;
          ::DBus::MethodProxy<void,std::string,uint16_t>::pointer m_method_set_remote_address_vsq;
          ::DBus::MethodProxy<void>::pointer m_method_unset_remote_address_v;
          ::DBus::MethodProxy<void,uint32_t,uint16_t>::pointer m_method_set_local_address_vuq;
          ::DBus::MethodProxy<void,std::string,uint16_t>::pointer m_method_set_local_address_vsq;


      };

    }
  }
}

#endif
