#ifndef __DBUS_PROXY_CONEXUS_DBUS_IPV4_UDP_H
#define __DBUS_PROXY_CONEXUS_DBUS_IPV4_UDP_H

#include <dbus-cxx.h>

#include <conexus/ipv4_udp.h>

#include <conexus-dbus/ipv4_ip_proxy.h>


namespace Conexus {
  namespace DBus {
    namespace IPv4 {

      class UDPProxy : public IPProxy
      {
        protected:
          UDPProxy( ::DBus::Connection::pointer conn, const std::string& dest="Conexus.IPv4.UDP", const std::string& path="/Conexus/IPv4/UDP"):
            IPProxy(conn, dest, path)
          {
            ::DBus::Path child_path;

            m_method_set_write_without_connect_vb = this->create_method< void,bool >( "Conexus.IPv4.UDP", "set_write_without_connect" );
            m_method_write_without_connect_b = this->create_method< bool >( "Conexus.IPv4.UDP", "write_without_connect" );
            m_method_add_multicast_interface_vu = this->create_method< void,uint32_t >( "Conexus.IPv4.UDP", "add_multicast_interface" );
            m_method_add_multicast_interface_vs = this->create_method< void,std::string >( "Conexus.IPv4.UDP", "add_multicast_interface" );
            m_method_remove_multicast_interface_vu = this->create_method< void,uint32_t >( "Conexus.IPv4.UDP", "remove_multicast_interface" );
            m_method_remove_multicast_interface_vs = this->create_method< void,std::string >( "Conexus.IPv4.UDP", "remove_multicast_interface" );
            m_method_is_multicast_interface_bu = this->create_method< bool,uint32_t >( "Conexus.IPv4.UDP", "is_multicast_interface" );
            m_method_is_multicast_interface_bs = this->create_method< bool,std::string >( "Conexus.IPv4.UDP", "is_multicast_interface" );
            m_method_set_suppress_multicast_join_vb = this->create_method< void,bool >( "Conexus.IPv4.UDP", "set_suppress_multicast_join" );
            m_method_suppress_multicast_join_b = this->create_method< bool >( "Conexus.IPv4.UDP", "suppress_multicast_join" );
            m_method_set_multicast_hop_limit_vu = this->create_method< void,uint32_t >( "Conexus.IPv4.UDP", "set_multicast_hop_limit" );
            m_method_multicast_hop_limit_u = this->create_method< uint32_t >( "Conexus.IPv4.UDP", "multicast_hop_limit" );
            m_method_set_default_multicast_hop_limit_vu = this->create_method< void,uint32_t >( "Conexus.IPv4.UDP", "set_default_multicast_hop_limit" );
            m_method_default_multicast_hop_limit_u = this->create_method< uint32_t >( "Conexus.IPv4.UDP", "default_multicast_hop_limit" );
            m_method_is_multicast_loopback_enabled_b = this->create_method< bool >( "Conexus.IPv4.UDP", "is_multicast_loopback_enabled" );
            m_method_enable_multicast_loopback_vb = this->create_method< void,bool >( "Conexus.IPv4.UDP", "enable_multicast_loopback" );
            m_method_is_default_multicast_loopback_enabled_b = this->create_method< bool >( "Conexus.IPv4.UDP", "is_default_multicast_loopback_enabled" );
            m_method_enable_default_multicast_loopback_vb = this->create_method< void,bool >( "Conexus.IPv4.UDP", "enable_default_multicast_loopback" );
          }

        public:

          typedef DBusCxxPointer<UDPProxy> pointer;

          static pointer create( ::DBus::Connection::pointer conn,
                                 const std::string& dest="Conexus.IPv4.UDP",
                                 const std::string& path="/Conexus/IPv4/UDP"
                               )
          { return pointer( new UDPProxy(conn, dest, path)); }


          void set_write_without_connect( bool value ) { return (*m_method_set_write_without_connect_vb)( value); }

          bool write_without_connect( ) { return (*m_method_write_without_connect_b)(); }

          void add_multicast_interface( uint32_t iface_index ) { return (*m_method_add_multicast_interface_vu)( iface_index); }

          void add_multicast_interface( std::string iface_index ) { return (*m_method_add_multicast_interface_vs)( iface_index); }

          void remove_multicast_interface( uint32_t iface_index ) { return (*m_method_remove_multicast_interface_vu)( iface_index); }

          void remove_multicast_interface( std::string iface_index ) { return (*m_method_remove_multicast_interface_vs)( iface_index); }

          bool is_multicast_interface( uint32_t iface_index ) { return (*m_method_is_multicast_interface_bu)( iface_index); }

          bool is_multicast_interface( std::string iface_index ) { return (*m_method_is_multicast_interface_bs)( iface_index); }

          void set_suppress_multicast_join( bool value ) { return (*m_method_set_suppress_multicast_join_vb)( value); }

          bool suppress_multicast_join( ) { return (*m_method_suppress_multicast_join_b)(); }

          void set_multicast_hop_limit( uint32_t limit ) { return (*m_method_set_multicast_hop_limit_vu)( limit); }

          uint32_t multicast_hop_limit( ) { return (*m_method_multicast_hop_limit_u)(); }

          void set_default_multicast_hop_limit( uint32_t limit ) { return (*m_method_set_default_multicast_hop_limit_vu)( limit); }

          uint32_t default_multicast_hop_limit( ) { return (*m_method_default_multicast_hop_limit_u)(); }

          bool is_multicast_loopback_enabled( ) { return (*m_method_is_multicast_loopback_enabled_b)(); }

          void enable_multicast_loopback( bool enabled ) { return (*m_method_enable_multicast_loopback_vb)( enabled); }

          bool is_default_multicast_loopback_enabled( ) { return (*m_method_is_default_multicast_loopback_enabled_b)(); }

          void enable_default_multicast_loopback( bool enabled ) { return (*m_method_enable_default_multicast_loopback_vb)( enabled); }


        protected:


          ::DBus::MethodProxy<void,bool>::pointer m_method_set_write_without_connect_vb;
          ::DBus::MethodProxy<bool>::pointer m_method_write_without_connect_b;
          ::DBus::MethodProxy<void,uint32_t>::pointer m_method_add_multicast_interface_vu;
          ::DBus::MethodProxy<void,std::string>::pointer m_method_add_multicast_interface_vs;
          ::DBus::MethodProxy<void,uint32_t>::pointer m_method_remove_multicast_interface_vu;
          ::DBus::MethodProxy<void,std::string>::pointer m_method_remove_multicast_interface_vs;
          ::DBus::MethodProxy<bool,uint32_t>::pointer m_method_is_multicast_interface_bu;
          ::DBus::MethodProxy<bool,std::string>::pointer m_method_is_multicast_interface_bs;
          ::DBus::MethodProxy<void,bool>::pointer m_method_set_suppress_multicast_join_vb;
          ::DBus::MethodProxy<bool>::pointer m_method_suppress_multicast_join_b;
          ::DBus::MethodProxy<void,uint32_t>::pointer m_method_set_multicast_hop_limit_vu;
          ::DBus::MethodProxy<uint32_t>::pointer m_method_multicast_hop_limit_u;
          ::DBus::MethodProxy<void,uint32_t>::pointer m_method_set_default_multicast_hop_limit_vu;
          ::DBus::MethodProxy<uint32_t>::pointer m_method_default_multicast_hop_limit_u;
          ::DBus::MethodProxy<bool>::pointer m_method_is_multicast_loopback_enabled_b;
          ::DBus::MethodProxy<void,bool>::pointer m_method_enable_multicast_loopback_vb;
          ::DBus::MethodProxy<bool>::pointer m_method_is_default_multicast_loopback_enabled_b;
          ::DBus::MethodProxy<void,bool>::pointer m_method_enable_default_multicast_loopback_vb;


      };

    }
  }
}

#endif
